/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// cconfig.cpp
//

#define _CCONFIG_CPP_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cconfig.h"
#include "fips.h"
#include "makeuc.h"
#include "datastruct.h"
#include "logmsg.h"

comconf g_ComConfig;

// defaults

#define DEFAULT_APRS_TO     "APRS"
#define DEFAULT_DIGIS       "WIDE3-3"

// init names, types, variables

#define STRING  0
#define CHAR    1
#define BOOLEAN 2
#define INTEGER 3
#define SHORT   4
#define UNS_INT 5
#define TIME    6
#define LONG    7
#define MULTISZ 8
#define FLINT   9
#define FLOAT   10

struct ini ComIni[] =
{
    { "ServerCall",         STRING,     &g_ComConfig.szServerCall },
    { "Alias",              STRING,     &g_ComConfig.szAlias },
    { "AprsToCall",         STRING,     &g_ComConfig.szAprsToCall },
    { "AprsDigis",          STRING,     &g_ComConfig.szAprsDigis },
    { "Ax25Port",           STRING,     &g_ComConfig.szAx25Port },
    { "Latitude",           FLOAT,      &g_ComConfig.dLatitude },
    { "Longitude",          FLOAT,      &g_ComConfig.dLongitude }
};

#define COM_INI_ENTRIES (sizeof(ComIni)/ sizeof(struct ini))



/////////////////////////////////////////////////////////////////////////////
// ReadComConfig - read <wxncommon.conf> to set required parameters.
// Returns TRUE success, FALSE failure.
//

BOOL ReadComConfig( void )
{
    FILE * hFile;
    BOOL bFound;
    BOOL bNoMatch;
    int i;
    char szString[STRING_SIZE];
    char szLine[STRING_SIZE];
    char szMsg[STRING_SIZE];
    char * pS;
    char * pParam;
    char * pVal;
    double dVal;


    hFile = fopen( WXN_SYSCONFDIR "wxncommon.conf", "r" );
    if( hFile == NULL )
    {
        g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, "cannot open/read file" );
        return FALSE;
    }

    // set defaults

    memset( &g_ComConfig, 0, sizeof(struct comconf) );

    FOREVER
    //for(;;)
    {
        if( fgets( szLine, STRING_SIZE, hFile ) == NULL )
            break;      // EOF

        if( *szLine == '\r' || *szLine == '\n' )
            continue;               // empty line

        pS =strchr( szLine, 0 );
        if( *(pS-1) == '\r' || *(pS-1) == '\n' )
            *(pS-1) = 0;

        strcpy( szString, szLine );

        bFound = FALSE;
        bNoMatch = TRUE;

        pParam = szString;
        while( *pParam )
        {
            if( *pParam != ' ' )
                break;
            pParam++;
        }

        if( *pParam == '#' )      // comment
            continue;

        pVal = pParam;
        while( *pVal )
        {
            if( *pVal == ' ' )
            {
                *pVal++ = 0;        // terminate the parameter portion of string
                continue;
            }
            if( *pVal++ == '=' )
                break;
        }
        while( *pVal )
        {
            if( *pVal != ' ' )
                break;
            pVal++;
        }

        if( *pVal == '"' )          // check for enclosing quotes
            pVal++;
        if( strchr( pVal, '"' ) )
            *strchr( pVal, '"' ) = 0;

        if( *pVal == 0 )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, szMsg );
            continue;
        }

        for( i=0; i<(int)COM_INI_ENTRIES; i++ )
        {
            if( strcasecmp( pParam, ComIni[i].szParam ) == 0 )
            {
                bFound = TRUE;
                bNoMatch = FALSE;
                if( pVal == NULL )
                    break;
                switch( ComIni[i].nType )
                {
                    case STRING:
                        strcpy( (char *) ComIni[i].pType, pVal );
                        break;

                    case CHAR:
                        *(char *) ComIni[i].pType = *pVal;
                        break;

                    case BOOLEAN:
                        if( ((*pVal & 0x005f) == 'T') || ((*pVal & 0x005f) == 'Y') )
                            *((BOOL *) ComIni[i].pType) = TRUE;
                        else
                            *((BOOL *) ComIni[i].pType) = FALSE;
                        break;

                    case INTEGER:
                        *((int *) ComIni[i].pType) = atoi( pVal );
                        break;

                    case SHORT:
                        *((short *) ComIni[i].pType) = (short) atoi( pVal );
                        break;

                    case UNS_INT:
                        *((unsigned *) ComIni[i].pType) = (unsigned) atol( pVal );
                        break;

                    case TIME:
                        *((time_t *) ComIni[i].pType) = (time_t) atol( pVal );
                        break;

                    case LONG:
                        *((long *) ComIni[i].pType) = atol( pVal );
                        break;

                    case FLINT:
                        dVal = atof( pVal ) * 100;
                        *((int *) ComIni[i].pType) = (int) dVal;   // one-liner introduces error
                        break;

                    case FLOAT:
                        dVal = atof( pVal );
                        *((double *) ComIni[i].pType) = dVal;
                        break;

                    default:
                        bNoMatch = TRUE;
                        break;
                }
            }

            if( bFound )
                break;
        }

        if( bNoMatch )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, szMsg );
        }
    }

    fclose( hFile );

    if( *g_ComConfig.szServerCall == 0 )
    {
        g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, "no server call defined" );
        return FALSE;
    }
    else
        makeuc( g_ComConfig.szServerCall );

    if( (g_ComConfig.dLatitude==0 ) && (g_ComConfig.dLongitude==0) )
    {
        g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, "no coordinates defined" );
        return FALSE;
    }

    if( *g_ComConfig.szAx25Port == 0 )
    {
        g_log.PostMsg( MSG_WXNCOMMON_CONF_FAIL, "no ax25 port defined" );
        return FALSE;
    }

    if( *g_ComConfig.szAprsToCall == 0 )
        strcpy( g_ComConfig.szAprsToCall, DEFAULT_APRS_TO );
    else
        makeuc( g_ComConfig.szAprsToCall );

    if( *g_ComConfig.szAprsDigis == 0 )
        strcpy( g_ComConfig.szAprsDigis, DEFAULT_DIGIS );
    else
        makeuc( g_ComConfig.szAprsDigis );

    return TRUE;
}

