create user root;
create user web;
create user sysop;

drop table hourly;
create table hourly
(

        dtg_local  timestamp without time zone       not null,

        tcr     int2,
        wac_dir int2,
        wac_spd int2,
        bcr     int2,
        plh     int2,
        hcr     int2,
        emp     int4,
        uvc     int2,
        ist     int2,
        ish     int2,
        tsd     int2,
        tsd2    int2,
        rha     int2,

        constraint      hourly_pk primary key( dtg_local )

);

grant select on hourly to public, web;
grant all on hourly to root, sysop;

drop table daily;
create table daily
(

        this_date       date    not null,

        thr_val         int2,
        thr_dtg         timestamp without time zone,
        tlr_val         int2,
        tlr_dtg         timestamp without time zone,
        mtemp           int2,
        heat_days       int2,
        cool_days       int2,

        bhr_val         int2,
        bhr_dtg         timestamp without time zone,
        blr_val         int2,
        blr_dtg         timestamp without time zone,
        mbar            int2,

        wgh_dir         int2,
        wgh_spd         int2,
        wgh_dtg         timestamp without time zone,
        mwind_spd       int2,
        mwind_dir       int2,

        hhr_val         int2,
        hhr_dtg         timestamp without time zone,
        hlr_val         int2,
        hlr_dtg         timestamp without time zone,
        mhumid          int2,

        precip          int2,

        edc             int4,

        uvh_val         int2,
        uvh_dtg         timestamp without time zone,
        uvl_val         int2,
        uvl_dtg         timestamp without time zone,
        muv             int2,

        mtsd            int2,
        mtsd2           int2,

        rda             int2,
        rmax            int2,
        ralarm_count    int2,

        constraint      daily_pk primary key( this_date )

);

grant select on daily to public, web;
grant all on daily to root, sysop;


drop table monthly;
create table monthly
(

        mo_date         date    not null,

        mtemp           int2,
        mwind_dir       int2,
        mwind_spd       int2,
        mbar            int2,
        mhumid          int2,
        precip          int2,
        emp             int4,
        muv             int2,
        mrad            int2,
        mtsd            int2,
        mtsd2           int2,
        heat_days       int2,
        cool_days       int2,

        constraint      monthly_pk primary key( mo_date )

);

grant select on monthly to public, web;
grant all on monthly to root, sysop;


drop table nodes;
create table nodes
(

    call_ssid       varchar(10) not null,
    alias           varchar(10),
    city            varchar(64),
    county          varchar(64),
    state           varchar(3),
    fips            varchar(7),
    afos            varchar(7),
    latitude        float8,
    longitude       float8,
    timezone        int2,
    elevation       int2,
    haat            int2,
    aprs_soft       int2,
    stn_type        int2,
    dst             bool,

    dtg_local       timestamp without time zone,
    dtg_zulu        timestamp without time zone,

    tcr             int2,
    thr_val         int2,
    thr_dtg         timestamp without time zone,
    tlr_val         int2,
    tlr_dtg         timestamp without time zone,
    trh             int2,
    trd             int2,
    tyh             int2,
    tyl             int2,

    tsd             int2,
    tsd2            int2,

    wca             int2,
    wcg             int2,

    cool_days       int2,
    heat_days       int2,

    wac_dir         int2,
    wac_spd         int2,
    wgc_dir         int2,
    wgc_spd         int2,

    wah_dir         int2,
    wah_spd         int2,
    wah_dtg         timestamp without time zone,
    wal_dir         int2,
    wal_spd         int2,
    wal_dtg         timestamp without time zone,

    wgh_dir         int2,
    wgh_spd         int2,
    wgh_dtg         timestamp without time zone,

    pca             int2,
    pri             int2,
    prh             int2,
    plh             int2,
    p24             int2,
    pty             int2,
    ptm             int2,
    pta             int2,

    bcr             int2,
    bhr_val         int2,
    bhr_dtg         timestamp without time zone,
    blr_val         int2,
    blr_dtg         timestamp without time zone,
    brh             int2,
    brd             int2,

    hcr             int2,
    hhr_val         int2,
    hhr_dtg         timestamp without time zone,
    hlr_val         int2,
    hlr_dtg         timestamp without time zone,
    hrh             int2,
    hrd             int2,

    fog             bool,
    heatindex       int2,
    dewpoint        int2,

    ist             int2,
    ish             int2,

    ecc             int4,
    eqc             int4,
    exc             int4,
    ehc             int4,
    edc             int4,

    rca             int2,
    rha             int2,
    rda             int2,
    rmax            int2,
    ralarm_count    int2,
    ralarm_trip     int2,

    uvc             int2,
    uvh_val         int2,
    uvh_dtg         timestamp without time zone,
    uvl_val         int2,
    uvl_dtg         timestamp without time zone,

    sun_now         bool,
    sun_min         int2,

    sunrise         timestamp without time zone,
    sunset          timestamp without time zone,

    alert           bool,

    constraint  nodes_pk primary key( call_ssid )

);

create index nodes_idx on nodes( dtg_zulu );

grant select on nodes to public, web;
grant all on nodes to root, sysop;


drop table emp;
create table emp
(

    dtg_local       timestamp without time zone  not null,
    dtg_zulu        timestamp without time zone  not null,
    ecount          int4    not null,

    constraint  emp_pk primary key( dtg_local )

);

grant select on emp to public, web;
grant all on emp to root, sysop;


drop table rcvtimes;
create table rcvtimes
(

    call_ssid    varchar(10) not null,

	aprs_time   timestamp without time zone,
	a_time      timestamp without time zone,
	b_time      timestamp without time zone,
	c_time      timestamp without time zone,
	d_time      timestamp without time zone,
	e_time      timestamp without time zone,

    constraint  rcvtimes_pk primary key( call_ssid )

);

grant select on rcvtimes to public, web;
grant all on rcvtimes to root, sysop;


drop table records;
create table records
(

    call_ssid   varchar(10) not null,

    htemp           int2,
    htemp_date      date,
    ltemp           int2,
    ltemp_date      date,

    wgh_spd         int2,
    wgh_dir         int2,
    wgh_date        date,

    precip24        int2,
    precip24_date   date,
    precipmh        int2,
    precipmh_date   date,
    precipml        int2,
    precipml_date   date,
    precipyh        int2,
    precipyh_date   date,
    precipyl        int2,
    precipyl_date   date,

    bhr             int2,
    bhr_date        date,
    blr             int2,
    blr_date        date,

    hemp            int4,
    hemp_date       date,

    hrad            int2,
    hrad_date       date,

    huv             int2,
    huv_date        date,

    constraint  records_pk primary key( call_ssid )

);

grant select on records to public, web;
grant all on records to root, sysop;

drop function populate( varchar, varchar );
create function populate( varchar, varchar ) returns bool as '
    declare
        h int2;
        m int2;
        m5 int2;
    begin
        for h in 0 .. 23 loop
            for m in 0 .. 11 loop
            m5 := m*5;
            execute ''insert into '' || $1 || ''(hour,min,'' || $2 ||
                '') values(''|| h ||'',''|| m5 ||'',-999)'';
            end loop;
        end loop;
        return true;
    end;
' language 'plpgsql';

drop table rad;
create table rad
(

    "hour"   int2    not null,
    min      int2    not null,
    rlt      int2    not null,

    constraint  rad_pk primary key( hour,min )

);

select populate( 'rad', 'rlt' );

grant select on rad to public, web;
grant all on rad to root, sysop;


drop table precip;
create table precip
(

    "hour"      int2    not null,
    min         int2    not null,
    plt         int2    not null,

    constraint  precip_pk primary key( hour,min )

);

select populate( 'precip', 'plt' );

grant select on precip to public, web;
grant all on precip to root, sysop;


drop table barom;
create table barom
(

    "hour"      int2 not null,
    min         int2 not null,
    bcr         int2 not null,

    constraint  barom_pk primary key( hour,min )

);

select populate( 'barom', 'bcr' );

grant select on barom to public, web;
grant all on barom to root, sysop;


drop table humid;
create table humid
(

    "hour"      int2 not null,
    min         int2 not null,
    hcr         int2 not null,

    constraint  humid_pk primary key( hour,min )

);

select populate( 'humid', 'hcr' );

grant select on humid to public, web;
grant all on humid to root, sysop;


drop table temp;
create table temp
(

    "hour"      int2 not null,
    min         int2 not null,
    tcr         int2 not null,

    constraint  temp_pk primary key( hour,min )

);

select populate( 'temp', 'tcr' );

grant select on temp to public, web;
grant all on temp to root, sysop;

drop function setwind();
create function setwind() returns bool as '
    declare
        h int2;
        m int2;
        m5 int2;
    begin
        for h in 0 .. 23 loop
            for m in 0 .. 11 loop
            m5 := m*5;
            execute ''insert into wind (hour,min) values(''|| h ||'',''|| m5 ||'')'';
            end loop;
        end loop;
        return true;
    end;
' language 'plpgsql';

drop table wind;
create table wind
(

    "hour"      int2 not null,
    min         int2 not null,
    wgc_dir     int2,
    wgc_spd     int2,
    wac_dir     int2,
    wac_spd     int2,

    constraint  wind_pk primary key( hour,min )

);

select setwind();

grant select on wind to public, web;
grant all on wind to root, sysop;


drop table queryd;
create table queryd
(

    dtg_local   timestamp without time zone not null,
    call_ssid   varchar(10) not null,
    query_cmd   varchar(16),
    node        varchar(10),

    constraint  queryd_pk primary key( dtg_local,call_ssid )

);

grant select on queryd to public, web;
grant all on queryd to root, sysop;


drop table alarm;
create table alarm
(

    call_ssid       varchar(10) not null,
    dtg_local       timestamp without time zone  not null,
    dtg_zulu        timestamp without time zone  not null,

    wac_spd         int2,
    wac_dir         int2,
    wgc_spd         int2,
    wgc_dir         int2,
    pca             int2,
    pri             int2,
    prh             int2,
    p24             int2,
    tcr             int2,
    bcr             int2,
    hcr             int2,
    uvc             int2,
    ecc             int4,
    eqc             int4,
    exc             int4,
    ehc             int4,
    edc             int4,
    rca             int2,
    rha             int2,
    rda             int2,
    rmax            int2,
    ralarm_count    int2,
    ralarm_trip     int2,

    constraint      alarm_pk primary key( dtg_zulu )

);

grant select on alarm to public, web;
grant all on alarm to root, sysop;


drop table alarmpoints;
create table alarmpoints
(

        call_ssid varchar(10) not null,

        pri     int2,
        prh     int2,
        p24     int2,
        wgh     int2,
        wah     int2,
        thr     int2,
        tlr     int2,
        blr     int2,
        ecc     int4,
        rad     int2,

        constraint      alarmpoints_pk primary key( call_ssid )

);

grant select on alarmpoints to public, web;
grant all on alarmpoints to root, sysop;


drop table email;
create table email
(

        recip   varchar(32) not null,
        host    varchar(32) not null,
        name    varchar(32),
        mtype   char(1) not null,
        alerts  varchar(16),

        constraint      email_pk primary key( recip, host )

);

grant all on email to root, sysop;

