/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//////////////////////////////////////////////////////////////////////////////
// aprproc - process aprs wx nodes
//
//                           -------------------
//  begin                : Wed Feb  5 11:47:20 CST 2003
//  copyright            : (C) 2003 by John Bennett
//  email                : n4xi@arrl.net
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
////////////////////////////////////////////////////////////////////////////

#define _APRPROC_CPP_

#include <string.h>
#include <stdlib.h>
#include <unistd.h>

// libwxn
#include <tz.h>
#include <logmsg.h>
#include <wxcalc.h>
#include <sun.h>
#include <fips.h>
#include <cconfig.h>

#include "aprsproc.h"
#include "aprs.h"
#include "qrz.h"
#include "deny.h"
#include "node.h"
#include "aglobal.h"
#include "aconfig.h"
#include "aprsqth.h"


// local function prototype
void SetNullTimestamps( noderx * pBlock );
void TransferData( noderx * pIn, noderx * pStore );
void AprsRxNode( noderx * pRx, noderx * pNew );
BOOL ValidateWxData( noderx * pNode );

char g_szUnknownQth[] = "<unknown>";


// for use by random number algorithm in Wxn extended data transmission
int g_nDelayMin;
int g_nDelayMax;
int g_nExtraTimeOffset;
int g_nAprsMinute;
int g_nAprsSecond;
int g_nUiFrameCount;


// deny class

deny g_denyNode;



//////////////////////////////////////////////////////////////////////////////
// AprsReceive - call to this function must immediately follow a status that
// indicates an APRS packet with weather data has been received.
//

void AprsReceive( void )
{
    noderx NodeIn;
    BOOL bZone;
    BOOL bSet;
    char szString[128];

    memset( &NodeIn, 0, NODERX_SIZE );
    memset( &g_RxNode, 0, NODERX_SIZE );

    // retrieve current data

    if( g_aprs.rxData( &NodeIn, g_Time.Utc() ) )
    {
        if( g_denyNode.check( NodeIn.Info.szCallSsid ) )
            return;

        if( !ValidateWxData( &NodeIn ) )
        {
            g_log.PostMsg( APRMSG_INVALID_RXDATA, NodeIn.Info.szCallSsid );
            return;
        }

        SetNullTimestamps( &NodeIn );

        //////////////////////////////////////// wind chill
        // Average
        NodeIn.Data.wca = Chill( NodeIn.Data.tcr, NodeIn.Data.wac.spd );
        // Gust
        NodeIn.Data.wcg = Chill( NodeIn.Data.tcr, NodeIn.Data.wgc.spd );

        //////////////////////////////////////// dew point/heat index

        if( NodeIn.Data.hcr )
        {
            NodeIn.Data.nDewPoint = DewPoint( NodeIn.Data.tcr, NodeIn.Data.hcr );
            if( NodeIn.Data.tcr < 68 )
                NodeIn.Data.nHeatIndex = 0;
            else
                NodeIn.Data.nHeatIndex = HeatIndex( NodeIn.Data.tcr, NodeIn.Data.hcr );
        }
        else
        {
            NodeIn.Data.nDewPoint = 0;
            NodeIn.Data.nHeatIndex = 0;
        }

        strcpy( g_RxNode.Info.szCallSsid, NodeIn.Info.szCallSsid );

        if( NodeExists( &g_RxNode ) )
        {
            // copy new information to current record
            g_RxNode.Info.nStnType = NodeIn.Info.nStnType;
            g_RxNode.Info.nAprSoftType = NodeIn.Info.nAprSoftType;
            g_RxNode.Data.tcr = NodeIn.Data.tcr;
            g_RxNode.Data.plh = NodeIn.Data.plh;
            g_RxNode.Data.p24 = NodeIn.Data.p24;
            g_RxNode.Data.pca = NodeIn.Data.pca;
            g_RxNode.Data.bcr = NodeIn.Data.bcr;
            g_RxNode.Data.hcr = NodeIn.Data.hcr,
            g_RxNode.Data.wgc.spd = NodeIn.Data.wgc.spd;
            g_RxNode.Data.wgc.dir = NodeIn.Data.wgc.dir;
            g_RxNode.Data.wac.spd = NodeIn.Data.wac.spd;
            g_RxNode.Data.wac.dir = NodeIn.Data.wac.dir;

            bZone = ReadZoneInfo( &g_RxNode );
            if( !bZone )
                g_log.PostMsg( APRMSG_ZONE_INFO_ERROR, NULL );

            if( g_aprs.Position() )
            {
                g_RxNode.Info.dLatitude = g_aprs.rxLatitude();
                g_RxNode.Info.dLongitude = g_aprs.rxLongitude();
            }

            if( !WriteBase( &g_RxNode ) )
                g_log.PostMsg( APRMSG_WRITEBASE_ERROR, NULL );

            CalculateFields( &g_RxNode.Data );
            if( bZone )
                CalculateRiseSet( &g_RxNode.Info, &g_RxNode.Data );
            if( !WriteCalc( &g_RxNode ) )
                g_log.PostMsg( APRMSG_WRITECALC_ERROR, NULL );

            return;
        }

        //////////////////// a new node

        // new nodes get default alias of call-ssid
        strcpy( NodeIn.Info.szAlias, NodeIn.Info.szCallSsid );

        // attempt to find in file <aprsqth> and resolve
        bSet = AprsQth( &NodeIn.Info );
        // if that didn't work, try callbook and then qrz
        if( !bSet && g_AprConfig.bUseCallbook )
            bSet = g_callbook.Lookup( &NodeIn.Info );
        // attempt to use QRZ to resolve
        else if( !bSet && g_AprConfig.bUseQrz )
            bSet = g_Qrz.QthLookup( &NodeIn.Info );

        if( bSet )
        {
            if( !GetFips( &NodeIn.Info ) )
            {
                sprintf( szString, "%s (%s), %s", NodeIn.Info.szCity, NodeIn.Info.szCounty, NodeIn.Info.szState );
                g_log.PostMsg( APRMSG_FIPS_LOOKUP_ERROR, szString );
                strcpy( NodeIn.Info.szFips, "00000" );
            }
        }
        // else unknown
        else
        {
            strcpy( NodeIn.Info.szCity, g_szUnknownQth );
            strcpy( NodeIn.Info.szFips, "00000" );
        }

        AprsRxNode( &NodeIn, &g_RxNode );

        return;

    }
    else
        g_log.PostMsg( APRMSG_RXDATA_FAIL_UNKNOWN, NULL );
}


/////////////////////////////////////////////////////////////////////////////
//  AprsRxNode - Add new APRS entry to database. APRS flags are set and
//  APRS specific data is copied from data to correct structures.
//

void AprsRxNode( noderx * pRx, noderx * pNew )
{
    memcpy( &pNew->Data, &pRx->Data, NODEDATA_SIZE );

    if( g_aprs.Position() )
    {
        pNew->Info.dLatitude = g_aprs.rxLatitude();
        pNew->Info.dLongitude = g_aprs.rxLongitude();
    }

    pNew->Info.tUpdate = g_Time.Utc();
    strcpy( pNew->Info.szAlias, pRx->Info.szAlias );
    strcpy( pNew->Info.szCallSsid, pRx->Info.szCallSsid );
    strcpy( pNew->Info.szCity, pRx->Info.szCity );
    strcpy( pNew->Info.szFips, pRx->Info.szFips );
    pNew->Info.nAprSoftType = pRx->Info.nAprSoftType;
    pNew->Info.nStnType = pRx->Info.nStnType;

    if( strcmp( pNew->Info.szFips, "00000" ) )
    {
        // only do this if FIPS is known
        if( !GetGeoData( pNew ) )
            return;
    }

    if( !CreateRecord( pNew ) )
        return;

    if( !WriteBase( pNew ) )
        g_log.PostMsg( APRMSG_WRITEBASE_ERROR, NULL );

    CalculateFields( &pNew->Data );
    if( pNew->Info.nTimeZone )
        CalculateRiseSet( &pNew->Info, &pNew->Data );
    if( !WriteCalc( pNew ) )
        g_log.PostMsg( APRMSG_WRITECALC_ERROR, NULL );
}


//////////////////////////////////////////////////////////////////////////////
// AprsPosition - call to this function must immediately follow a status that
// indicates an APRS packet with position data has been received.
//

void AprsPosition( void )
{
    position pos;

    if( g_aprs.rxPos( &pos ) )
    {
        g_aprs.RxFmCall( g_RxNode.Info.szCallSsid );
        if( ReadNode( &g_RxNode ) )       // retrieve record
        {
            // fill information
            g_RxNode.Info.dLatitude = pos.dLat;
            g_RxNode.Info.dLongitude = pos.dLng;

            // write it back
            if( !WriteBase( &g_RxNode ) )
                g_log.PostMsg( APRMSG_NODES_UPDATE_FAIL, NULL );
        }

        else
            // can only process position data for nodes in table
            return;
    }
}


//////////////////////////////////////////////////////////////////////////////
// AprsWxnData - receive data packet from another WXN server via APRS.
//

void AprsWxnData( void )
{
    noderx NodeIn;
    noderx g_RxNode;
    time_t tTime;
    BOOL bStatus;
    char szString[128];

    memset( &NodeIn, 0, NODERX_SIZE );
    SetNullTimestamps( &NodeIn );

    memset( &g_RxNode, 0, NODERX_SIZE );

    tTime = g_Time.Utc();
    if( g_aprs.rxData( &NodeIn, tTime ) )
    {
        if( g_denyNode.check( NodeIn.Info.szCallSsid ) )
            return;

        strcpy( g_RxNode.Info.szCallSsid, NodeIn.Info.szCallSsid );

        if( ReadNode( &g_RxNode ) )
        {
            // fill information
            g_RxNode.Info.tUpdate = g_Time.Utc();
            strcpy( g_RxNode.Info.szAlias, NodeIn.Info.szAlias );
            strcpy( g_RxNode.Info.szCallSsid, NodeIn.Info.szCallSsid );

            TransferData( &NodeIn, &g_RxNode );
            CalculateFields( &g_RxNode.Data );

            // TYPE_A packet
            if( g_aprs.DecodeType() == TYPE_A )
            {
                // only TYPE_A contains node alias
                if( !GetGeoData( &g_RxNode ) )
                {
                    sprintf( szString, "WXN extended decode [%s]", g_RxNode.Info.szCallSsid );
                    g_log.PostMsg( APRMSG_GETGEODATA_FAIL, szString );
                    return;
                }
                CalculateRiseSet( &g_RxNode.Info, &g_RxNode.Data );
            }

            switch( g_aprs.DecodeType() )
            {
                case TYPE_A:
                    bStatus = WriteTypeA( &g_RxNode );
                    break;

                case TYPE_B:
                    bStatus = WriteTypeB( &g_RxNode );
                    break;

                case TYPE_C:
                    bStatus = WriteTypeC( &g_RxNode );
                    break;

                case TYPE_D:
                    bStatus = WriteTypeD( &g_RxNode );
                    break;

                case TYPE_E:
                    bStatus = WriteTypeE( &g_RxNode );
                    break;

                default:
                    bStatus = FALSE;
                    break;
            }

            if( !bStatus )
                g_log.PostMsg( APRMSG_NODES_UPDATE_FAIL, "(WXN extended data)" );
        }
        // else must wait for initial APRS data packet
    }
}


void SetNullTimestamps( noderx * pBlock )
{
    // set timestamps to error (i.e., no time)

    pBlock->Data.bhr.dtg = -1;
    pBlock->Data.blr.dtg = -1;
    pBlock->Data.thr.dtg = -1;
    pBlock->Data.tlr.dtg = -1;
    pBlock->Data.hhr.dtg = -1;
    pBlock->Data.hlr.dtg = -1;
    pBlock->Data.wah.dtg = -1;
    pBlock->Data.wal.dtg = -1;
    pBlock->Data.wgh.dtg = -1;
}


////////////////////////////////////////////////
// TransferData - transfers data from parsed
// received data to global block for storage.
//

void TransferData( noderx * pIn, noderx * pStore )
{
    if( g_aprs.DecodeType() == TYPE_A )
    {
        strcpy( pStore->Info.szFips, pIn->Info.szFips );
        strcpy( pStore->Info.szCity, pIn->Info.szCity );
        strcpy( pStore->Info.szAlias, pIn->Info.szAlias );
        pStore->Data.tcr = pIn->Data.tcr,
        pStore->Data.thr.val = pIn->Data.thr.val;
        pStore->Data.thr.dtg = pIn->Data.thr.dtg;
        pStore->Data.tlr.val = pIn->Data.tlr.val;
        pStore->Data.tlr.dtg = pIn->Data.tlr.dtg;
        pStore->Data.trd = pIn->Data.trd;
        pStore->Data.trh = pIn->Data.trh;
        pStore->Data.tyh = pIn->Data.tyh;
        pStore->Data.tyl = pIn->Data.tyl;
        pStore->Data.tsd = pIn->Data.tsd;
    }

    else if(g_aprs.DecodeType() == TYPE_B )
    {
        pStore->Data.pca = pIn->Data.pca;
        pStore->Data.pri = pIn->Data.pri;
        pStore->Data.prh = pIn->Data.prh;
        pStore->Data.plh = pIn->Data.plh;
        pStore->Data.p24 = pIn->Data.p24;
        pStore->Data.pty = pIn->Data.pty;
        pStore->Data.ptm = pIn->Data.ptm;
        pStore->Data.pta = pIn->Data.pta;
        pStore->Data.ecc = pIn->Data.ecc;
        pStore->Data.eqc = pIn->Data.eqc;
        pStore->Data.exc = pIn->Data.exc;
        pStore->Data.ehc = pIn->Data.ehc;
        pStore->Data.edc = pIn->Data.edc;
    }

    else if(g_aprs.DecodeType() == TYPE_C )
    {
        pStore->Data.wac.spd = pIn->Data.wac.spd;
        pStore->Data.wac.dir = pIn->Data.wac.dir;
        pStore->Data.wgc.spd = pIn->Data.wgc.spd;
        pStore->Data.wgc.dir = pIn->Data.wgc.dir;
        pStore->Data.wah.spd = pIn->Data.wah.spd;
        pStore->Data.wah.dir = pIn->Data.wah.dir;
        pStore->Data.wah.dtg = pIn->Data.wah.dtg;
        pStore->Data.wal.spd = pIn->Data.wal.spd;
        pStore->Data.wal.dir = pIn->Data.wal.dir;
        pStore->Data.wal.dtg = pIn->Data.wal.dtg;
        pStore->Data.wgh.spd = pIn->Data.wgh.spd;
        pStore->Data.wgh.dir = pIn->Data.wgh.dir;
        pStore->Data.wgh.dtg = pIn->Data.wgh.dtg;
        pStore->Info.nHAAT = pIn->Info.nHAAT;
    }

    else if(g_aprs.DecodeType() == TYPE_D )
    {
        pStore->Data.bcr = pIn->Data.bcr;
        pStore->Data.bhr.val = pIn->Data.bhr.val;
        pStore->Data.bhr.dtg = pIn->Data.bhr.dtg;
        pStore->Data.blr.val = pIn->Data.blr.val;
        pStore->Data.blr.dtg = pIn->Data.blr.dtg;
        pStore->Data.brd = pIn->Data.brd;
        pStore->Data.brh = pIn->Data.brh;
        pStore->Data.hcr = pIn->Data.hcr,
        pStore->Data.hhr.val = pIn->Data.hhr.val;
        pStore->Data.hhr.dtg = pIn->Data.hhr.dtg;
        pStore->Data.hlr.val = pIn->Data.hlr.val;
        pStore->Data.hlr.dtg = pIn->Data.hlr.dtg;
        pStore->Data.hrd = pIn->Data.hrd;
        pStore->Data.hrh = pIn->Data.hrh;
        pStore->Data.ist = pIn->Data.ist;
        pStore->Data.ish = pIn->Data.ish;
    }

    else if(g_aprs.DecodeType() == TYPE_E )
    {
        pStore->Data.rca = pIn->Data.rca;
        pStore->Data.rha = pIn->Data.rha;
        pStore->Data.rda = pIn->Data.rda;
        pStore->Data.rmax = pIn->Data.rmax;
        pStore->Data.ralarm_count = pIn->Data.ralarm_count;
        pStore->Info.nRadAlarmTrip = pIn->Info.nRadAlarmTrip;
    }

    // else a screw-up

}



//////////////////////////////////////////////////////////////////////////////
// AprsRandomTimerInit - init random timer variables
//

void AprsRandomTimerInit( void )
{
    srand( (unsigned)time( NULL ) );

    g_nDelayMin = MIN_NORMAL_DELAY;
    g_nDelayMax = MAX_NORMAL_DELAY;

    // be sure we don't have a hissy if we haven't been on-line
    g_aprs.UiFrameCount();  // reading clears the current count
    g_nUiFrameCount = 0;

    g_nAprsMinute = g_Time.Minute() % HOST_INTERVAL;
    if( g_nAprsMinute > 2 )
        g_nAprsMinute -= 2;
    g_nAprsSecond = g_Time.Second();

    g_nExtraTimeOffset = g_Time.Minute();
    while( (g_nExtraTimeOffset-g_nAprsMinute) % HOST_INTERVAL )
        g_nExtraTimeOffset++;
}



//////////////////////////////////////////////////////////////////////////////
// SetAprsExtendedDelay - only take effect if delay threshold is non-zero

void SetAprsExtendedDelay( void )

{
    g_nUiFrameCount = g_aprs.UiFrameCount();
    if( g_AprConfig.nAprsDelayThreshold )
    {
        if( g_nUiFrameCount > g_AprConfig.nAprsDelayThreshold )
        {
            g_nDelayMin = MIN_NORMAL_DELAY;
            g_nDelayMax = MAX_NORMAL_DELAY;
        }
        else
        {
            g_nDelayMin = MIN_BUSY_DELAY;
            g_nDelayMax = MAX_BUSY_DELAY;
        }
    }
}


//////////////////////////////////////////////////////////////////////////////
// AprsRandomDelay - returns random delay time in seconds
//

time_t AprsRandomDelay( void )
{
    return g_nDelayMin + rand()%g_nDelayMax;
}



void AprsStnTypeInit( void )
{
    strcpy( g_Local.Info.szCallSsid, g_ComConfig.szServerCall );
    if( !ReadNode( &g_Local ) )
        return;

    g_aprs.SetStationType( g_Local.Info.nStnType );
}


//////////////////////////////////////////////////////////////////////////////
// ValidateWxData
//

BOOL ValidateWxData( noderx * pNode )
{
    if( (pNode->Data.tcr + pNode->Data.bcr + pNode->Data.hcr + pNode->Data.wgc.spd) == 0 )
        return FALSE;
    else
        return TRUE;
}


//////////////////////////////////////////////////////////////////////////////
// init 'deny'
//

void InitLists( int nSig )
{
    // if nSig is non-zero we were called by SIGHUP
    if( nSig )
        g_log.PostMsg( APRMSG_DENY_REINIT, NULL );

    if( !g_denyNode.init( "node" ) )
        g_log.PostMsg( APRMSG_DENY_INIT_FAIL, "node.deny" );
}

