/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// qrz.h - class interface to QRZ database
//

#ifndef _QRZ_H_
#define _QRZ_H_

// libwxn
#include <common.h>
#include <datastruct.h>


#define DATAFILE_SIZE       16
#define BYTESPERKEY_SIZE    8
#define NUMKEYS_SIZE        8
#define KEYLEN_SIZE         8
#define VERSION_SIZE        8
#define QRZ_BUFFER_SIZE     256

struct rawheader
{
    char datafile[DATAFILE_SIZE];       // name of the datafile
    char bytesperkey[BYTESPERKEY_SIZE]; // number of bytes in dbf for each index item
    char numkeys[NUMKEYS_SIZE];         // number of index items (keys) in file
    char keylen[KEYLEN_SIZE];           // length of each item (key)
    char version[VERSION_SIZE];         // db version id; end of "header"
};

struct header
{
    long bytesperkey;                   // number of bytes in dbf for each index item
    int numkeys;                        // number of index items (keys) in file
    int keylen;                         // length of each item (key)
};

struct qrzrecord
{
    char callsign[6+1];
    char lastname[32+1];
    char namesuffix[2+1];
    char firstname[32+1];
    char middleinit[1+1];
    char streetaddr[32+1];
    char city[32+1];
    char state[2+1];
    char datelicensed[5+1];
    char dob[5+1];
    char LicenseClass[1+1];
    char zipcode[5+1];
    char prevcall[6+1];
    char prevclass[1+1];
};


struct CallSign
{
    char prefix[4];
    char area[4];
    char suffix[4];
};



class qrz
{
    public:
        qrz();
        ~qrz();

        BOOL Init( char * pPath );
        BOOL QthLookup( noderef * pInfo );

    private:
        BOOL SetIndex( const char * szCall );
        BOOL ParseField( char * pDest, int nMax );
        void FormatCall( char * szCall );
        BOOL Search( const char * szCall );
        BOOL IsCrossRef( char *szCall );

        FILE * m_hIndex;
        FILE * m_hDbf;

        struct rawheader m_RawHeader;
        struct header m_Index;
        struct qrzrecord m_Record;

        char * m_pBuffer;

        char * m_pKeybase;
        size_t m_nKeysize;

        BOOL m_bEnabled;

        char m_szCallSsid[CALLSSID_SIZE+1];
        char m_szTarget[CALLSSID_SIZE+1];
        char m_szBuffer[QRZ_BUFFER_SIZE+1];
};



#endif      // _QRZ_H_

