/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define _ARCHIVE_CPP_

#include <stdlib.h>
#include <string.h>
#include <libpq-fe.h>

// libwxn
#include <tz.h>
#include <wxcalc.h>

#include "archive.h"
#include "hglobal.h"
#include "wxdata.h"
#include "precip.h"
#include "sensors.h"
//#include "rates.h"
#include "calendar.h"


int g_nMonthDays[] = { JAN_DAYS, FEB_DAYS, MAR_DAYS, APR_DAYS, MAY_DAYS, JUN_DAYS,
                       JUL_DAYS, AUG_DAYS, SEP_DAYS, OCT_DAYS, NOV_DAYS, DEC_DAYS };


////////////////////////////////////////////////////////////////////////////
// WriteHourly - write hourly data to backend SQL

void WriteHourly( void )
{

    PGconn * pConn;
    PGresult * pResult;
    char szExec[128];
    char szTime[64];

    g_Time.CurrentLocalTimestamp( szTime );

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return;
    }

    pResult = PQexec( pConn, "BEGIN" );
    PQclear( pResult );

    // create the record
    sprintf( szExec, "INSERT INTO hourly( dtg_local ) VALUES( '%s' )",
        szTime );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return;
    }
    PQclear( pResult );

    // temperature, wind, precip
    sprintf( szExec, "UPDATE hourly SET tcr=%i, wac_spd=%i, wac_dir=%i, plh=%i "
        "WHERE dtg_local='%s'",
        g_Local.Data.tcr, g_Local.Data.wac.spd, g_Local.Data.wac.dir, g_Local.Data.prh, // plh == prh on hour
        szTime );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // for the following sensors, must write only if sensor is attached

    // barometer
    if( g_Sensors.bBarom || g_Sensors.bOwBarom )
    {
        sprintf( szExec, "UPDATE hourly SET bcr=%i WHERE dtg_local='%s'",
            g_Local.Data.bcr, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // humidity
    if( g_Sensors.bOutsideHumid || g_Sensors.bOwOutsideHumid)
    {
        sprintf( szExec, "UPDATE hourly SET hcr=%i WHERE dtg_local='%s'",
            g_Local.Data.hcr, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // emp
    if( g_Sensors.bOwEmp )
    {
        sprintf( szExec, "UPDATE hourly SET emp=%i WHERE dtg_local='%s'",
            g_Local.Data.ehc, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // radiation - avg for last hour
    if( g_Sensors.bOwRad )
    {
        sprintf( szExec, "UPDATE hourly SET rha=%i WHERE dtg_local='%s'",
            g_Local.Data.rha, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // soil temp at 6" depth
    if( g_Sensors.bOwSoilTemp )
    {
        sprintf( szExec, "UPDATE hourly SET tsd=%i WHERE dtg_local='%s'",
            g_Local.Data.tsd, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // inside temp
    if( g_Sensors.bInsideTemp || g_Sensors.bInsideTemp )
    {
        sprintf( szExec, "UPDATE hourly SET ist=%i WHERE dtg_local='%s'",
            g_Local.Data.ist, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // inside humidity
    if( g_Sensors.bInsideHumid || g_Sensors.bOwInsideHumid )
    {
        sprintf( szExec, "UPDATE hourly SET ish=%i WHERE dtg_local='%s'",
            g_Local.Data.ish, szTime );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    pResult = PQexec( pConn, "COMMIT" );
    PQclear( pResult );

    PQfinish( pConn );

}


////////////////////////////////////////////////////////////////////////////
// WriteDaily - determine daily summary, write to daily table. Called
// after daily summary at midnight, but before hi/lo settings cleared.
//

void WriteDaily( void )
{
    PGconn * pConn;
    PGresult * pResult;
    char szExec[384];
    char szThisDate[16];
    char szHiTime[32];
    char szLoTime[32];
    int i;
    unsigned uAvgSpeed;
    unsigned uAvgDir;
    unsigned uWCount;
    int nHeatDays;
    int nCoolDays;
    int nTCount;
    int nHCount;
    int nSCount;
    int nBCount;
    int nMeanTemp;
    int nMeanHumid;
    int nMeanBarom;
    int nMeanSoilTemp;
    int nTemp[24];

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return;
    }

    sprintf( szThisDate, "%.4i-%.2i-%.2i", g_Time.PrevYear(), g_Time.PrevMonth(), g_Time.PrevDay() );

    // create the record
    sprintf( szExec, "INSERT INTO daily( this_date ) VALUES( '%s' )", szThisDate );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return;
    }
    PQclear( pResult );

    //////////////////////// find mean and median values as required

    // init variables
    for( i=0; i<24; i++ )
        nTemp[i] = 0;
    nTCount = 0;    // temp
    nHCount = 0;    // humidity
    nSCount = 0;    // soil temp
    nBCount = 0;    // barometer
    uWCount = 0;    // wind
    uAvgSpeed = 0;
    uAvgDir = 0;
    nMeanSoilTemp = 0;
    nMeanHumid = 0;
    nMeanBarom = 0;

    // get data
    for( i=0; i<24; i++ )
    {
        sprintf( szExec, "SELECT tcr,hcr,bcr,tsd,wac_spd,wac_dir FROM hourly "
            "WHERE date_trunc( 'hour', dtg_local ) = '%.4i-%.2i-%.2i %.2i:00'",
            g_Time.PrevYear(), g_Time.PrevMonth(), g_Time.PrevDay(), i );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )     // temperature
            {
                nTemp[nTCount] = atoi( PQgetvalue( pResult, 0, 0 ) );
                nTCount++;
            }
            if( !PQgetisnull( pResult, 0, 1 ) )     // humidity
            {
                nMeanHumid += atoi( PQgetvalue( pResult, 0, 1 ) );
                nHCount++;
            }
            if( !PQgetisnull( pResult, 0, 2 ) )     // barometer
            {
                nMeanBarom += atoi( PQgetvalue( pResult, 0, 2 ) );
                nBCount++;
            }
            if( !PQgetisnull( pResult, 0, 3 ) )     // soil temp
            {
                nMeanSoilTemp += atoi( PQgetvalue( pResult, 0, 3 ) );
                nSCount++;
            }
            if( !PQgetisnull( pResult, 0, 4 ) )     // avg wind speed/direction
            {
                uWCount++;
                uAvgSpeed += (unsigned) atoi( PQgetvalue( pResult, 0, 4 ) );
                uAvgDir += (unsigned) atoi( PQgetvalue( pResult, 0, 5 ) );
            }
        }
        PQclear( pResult );
    }

    //////////////////////// determine means

    if( nTCount )
    {
        // mean temperature
        nMeanTemp = 0;
        for( i=0; i<nTCount; i++ )
            nMeanTemp += nTemp[i];
        nMeanTemp /= nTCount;
    }
    else
    {
        nMeanTemp = 999;
    }

    // wind averages
    if( uWCount )
    {
        uAvgSpeed /= uWCount;
        uAvgDir /= uWCount;
    }

    if( nHCount )
        nMeanHumid /= nHCount;
    if( nBCount )
        nMeanBarom /= nBCount;
    if( nSCount )
        nMeanSoilTemp /= nSCount;

    nHeatDays = HeatDays( nMeanTemp );
    nCoolDays = CoolDays( nMeanTemp );

    //////////////////////// process data

    pResult = PQexec( pConn, "BEGIN" );
    PQclear( pResult );

    // temperature
    g_Time.MakeLocalTimestamp( szHiTime, g_Local.Data.thr.dtg );
    g_Time.MakeLocalTimestamp( szLoTime, g_Local.Data.tlr.dtg );
    sprintf( szExec, "UPDATE daily SET thr_val=%i, thr_dtg='%s', tlr_val=%i, tlr_dtg='%s', mtemp=%i "
        "WHERE this_date = '%s'",
        g_Local.Data.thr.val, szHiTime, g_Local.Data.tlr.val, szLoTime, nMeanTemp, szThisDate );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    ////////// degree days
    // daily table
    sprintf( szExec, "UPDATE daily SET heat_days=%i, cool_days=%i WHERE this_date = '%s'",
        nHeatDays, nCoolDays, szThisDate );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );
    // local data structure
    g_Local.Data.nHeatDays = nHeatDays;
    g_Local.Data.nCoolDays = nCoolDays;
    // nodes table
    sprintf( szExec, "UPDATE nodes SET heat_days=%i, cool_days=%i WHERE call_ssid = '%s'",
        nHeatDays, nCoolDays, g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // precip
    sprintf( szExec, "UPDATE daily SET precip=%i WHERE this_date = '%s'",
        g_Local.Data.pca, szThisDate );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    ////////////// wind
    // gust
    g_Time.MakeLocalTimestamp( szHiTime, g_Local.Data.wgh.dtg );
    sprintf( szExec, "UPDATE daily SET wgh_spd=%i, wgh_dir=%i, wgh_dtg='%s' WHERE this_date = '%s'",
        g_Local.Data.wgh.spd, g_Local.Data.wgh.dir, szHiTime, szThisDate );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    sprintf( szExec, "UPDATE daily SET mwind_spd=%u, mwind_dir=%u WHERE this_date = '%s'",
        uAvgSpeed, uAvgDir, szThisDate );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // for the following sensors, must write only if sensor is attached

    // barometer
    if( g_Sensors.bBarom || g_Sensors.bOwBarom )
    {
        g_Time.MakeLocalTimestamp( szHiTime, g_Local.Data.bhr.dtg );
        g_Time.MakeLocalTimestamp( szLoTime, g_Local.Data.blr.dtg );
        sprintf( szExec, "UPDATE daily SET bhr_val=%i, bhr_dtg='%s', blr_val=%i, blr_dtg='%s', mbar=%i "
            "WHERE this_date = '%s'",
            g_Local.Data.bhr.val, szHiTime, g_Local.Data.blr.val, szLoTime, nMeanBarom, szThisDate );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // humidity
    if( g_Sensors.bOutsideHumid || g_Sensors.bOwOutsideHumid )
    {
        g_Time.MakeLocalTimestamp( szHiTime, g_Local.Data.hhr.dtg );
        g_Time.MakeLocalTimestamp( szLoTime, g_Local.Data.hlr.dtg );
        sprintf( szExec, "UPDATE daily SET hhr_val=%i, hhr_dtg='%s', hlr_val=%i, hlr_dtg='%s', mhumid=%i "
            "WHERE this_date = '%s'",
            g_Local.Data.hhr.val, szHiTime, g_Local.Data.hlr.val, szLoTime, nMeanHumid, szThisDate );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // emp
    if( g_Sensors.bOwEmp )
    {
        sprintf( szExec, "UPDATE daily SET edc=%u WHERE this_date = '%s'",
            g_Local.Data.edc, szThisDate );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // radiation
    if( g_Sensors.bOwRad )
    {
        sprintf( szExec, "UPDATE daily SET rda=%i, rmax=%i, ralarm_count=%i "
            "WHERE this_date = '%s'",
            g_Local.Data.rda, g_Local.Data.rmax, g_Local.Data.ralarm_count,
            szThisDate );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // tsd
    if( g_Sensors.bOwSoilTemp )
    {
        sprintf( szExec, "UPDATE daily SET mtsd=%i WHERE this_date = '%s'",
            nMeanSoilTemp, szThisDate );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    pResult = PQexec( pConn, "COMMIT" );
    PQclear( pResult );

    PQfinish( pConn );
}


/////////////////////////////////////////////////////////////////////
// WriteMonthly - write monthly summary to pgsql.
// Called at midnight after daily write has been completed.
//

void WriteMonthly( void )
{
    int i;
    struct monthly month;
    char szExec[256];
    PGconn * pConn;
    PGresult * pResult;
    int nMonth;
    int nYear;

    // init values for averaging/totals
    memset( &month, 0, MONTHLY_SIZE );

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return;
    }

    nMonth = g_Time.PrevMonth();
    nYear = g_Time.PrevYear();

    for( i=0; i<g_nMonthDays[nMonth-1]; i++ )
    {
        // mean temperature
        sprintf( szExec, "SELECT mtemp FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mtemp.cnt++;
                month.mtemp.val += atoi( PQgetvalue( pResult, 0, 0 ) );
            }
            PQclear( pResult );
        }
        else
        {
            PQclear( pResult );
            continue;   // record for day does not exist
        }

        // degree days
        sprintf( szExec, "SELECT heat_days, cool_days FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.nHeatDays += atoi( PQgetvalue( pResult, 0, 0 ) );
                month.nCoolDays += atoi( PQgetvalue( pResult, 0, 1 ) );
            }
        }
        PQclear( pResult );

        // mean barometer
        sprintf( szExec, "SELECT mbar FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mbar.cnt++;
                month.mbar.val += (unsigned long) atoi( PQgetvalue( pResult, 0, 0 ) );
            }
        }
        PQclear( pResult );

        // mean humidity
        sprintf( szExec, "SELECT mhumid FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mhumid.cnt++;
                month.mhumid.val += atoi( PQgetvalue( pResult, 0, 0 ) );
            }
        }
        PQclear( pResult );

        // precip total
        sprintf( szExec, "SELECT precip FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
                month.precip += atoi( PQgetvalue( pResult, 0, 0 ) );
        }
        PQclear( pResult );

        // emp total
        sprintf( szExec, "SELECT edc FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
                month.emp += (unsigned long) atoi( PQgetvalue( pResult, 0, 0 ) );
        }
        PQclear( pResult );

        // radiation count avg
        sprintf( szExec, "SELECT rda FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mrad.cnt++;
                month.mrad.val += atoi( PQgetvalue( pResult, 0, 0 ) );
            }
        }
        PQclear( pResult );

        // avg soil temp at 6" depth
        sprintf( szExec, "SELECT mtsd FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mtsd.cnt++;
                month.mtsd.val += atoi( PQgetvalue( pResult, 0, 0 ) );
            }
        }
        PQclear( pResult );

        // avg wind speed/dir
        sprintf( szExec, "SELECT mwind_spd FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
            nYear, nMonth, i+1 );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) == PGRES_TUPLES_OK )
        {
            if( !PQgetisnull( pResult, 0, 0 ) )
            {
                month.mwind.spd += atoi( PQgetvalue( pResult, 0, 0 ) );
                PQclear( pResult );
                sprintf( szExec, "SELECT mwind_dir FROM daily WHERE this_date = '%.4i-%.2i-%.2i'",
                    nYear, nMonth, i+1 );
                pResult = PQexec( pConn, szExec );
                month.mwind.dir += atoi( PQgetvalue( pResult, 0, 0 ) );
                month.mwind.cnt++;
            }
        }
        PQclear( pResult );

    }   // end for(...)


    // create the record

    pResult = PQexec( pConn, "BEGIN" );
    PQclear( pResult );

    sprintf( szExec, "INSERT INTO monthly( mo_date ) VALUES( '%.4i-%.2i-01' )",
        nYear, nMonth );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return;
    }
    PQclear( pResult );

    // degree days, precip, emp
    sprintf( szExec, "UPDATE monthly SET heat_days=%i, cool_days=%i, precip=%i, emp=%lu WHERE mo_date = '%.4i-%.2i-01'",
        month.nHeatDays, month.nCoolDays, month.precip, month.emp, nYear, nMonth );
    pResult = PQexec( pConn, szExec );
    PQclear( pResult );

    // In the following, the field is only written to the record if a reading existed.
    // This will permit detecting when a sensor is not used (or functional) in the system.

    // temperature
    if( month.mtemp.cnt )
    {
        month.mtemp.val /= month.mtemp.cnt;
        sprintf( szExec, "UPDATE monthly SET mtemp=%i WHERE mo_date = '%.4i-%.2i-01'",
            month.mtemp.val, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // wind
    if( month.mwind.cnt )
    {
        month.mwind.spd /= month.mwind.cnt;
        month.mwind.dir /= month.mwind.cnt;
        sprintf( szExec, "UPDATE monthly SET mwind_spd=%i, mwind_dir=%i WHERE mo_date = '%.4i-%.2i-01'",
            month.mwind.spd, month.mwind.dir, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // barometer
    if( month.mbar.cnt )
    {
        month.mbar.val /= month.mbar.cnt;
        sprintf( szExec, "UPDATE monthly SET mbar=%i WHERE mo_date = '%.4i-%.2i-01'",
            (int) month.mbar.val, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // humidity
    if( month.mhumid.cnt )
    {
        month.mhumid.val /= month.mhumid.cnt;
        sprintf( szExec, "UPDATE monthly SET mhumid=%i WHERE mo_date = '%.4i-%.2i-01'",
            month.mhumid.val, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // radiation count
    if( month.mrad.cnt )
    {
        month.mrad.val /= month.mrad.cnt;
        sprintf( szExec, "UPDATE monthly SET mrad=%i WHERE mo_date = '%.4i-%.2i-01'",
            month.mrad.val, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    // soil temperature at 6" depth
    if( month.mtsd.cnt )
    {
        month.mtsd.val /= month.mtsd.cnt;
        sprintf( szExec, "UPDATE monthly SET mtsd=%i WHERE mo_date = '%.4i-%.2i-01'",
            month.mtsd.val, nYear, nMonth );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );
    }

    pResult = PQexec( pConn, "COMMIT" );
    PQclear( pResult );

    PQfinish( pConn );
}



/////////////////////////////////////////////////////////////////////
// SetWxRecords - determine and write record highs/lows to pgsql
//

void SetWxRecords( void )
{
    char szExec[384];
    char szDate[16];
    PGconn * pConn;
    PGresult * pResult;
    int nTemp;
    unsigned uTemp;

    pConn = PQconnectdb( "dbname=wxn" );
    if( pConn == NULL )
        return;
    if( PQstatus( pConn ) != CONNECTION_OK )
    {
        PQfinish( pConn );
        return;
    }

    sprintf( szDate, "%i-%.2i-%.2i", g_Time.PrevYear(), g_Time.PrevMonth(), g_Time.PrevDay() );

    // check for existence and create record if necessary
    sprintf( szExec, "SELECT call_ssid FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( PQresultStatus( pResult ) != PGRES_TUPLES_OK )
    {
        PQclear( pResult );
        PQfinish( pConn );
        return;
    }
    if( PQntuples( pResult ) == 0 )         // record does not exist
    {
        PQclear( pResult );

        pResult = PQexec( pConn, "BEGIN" );
        PQclear( pResult );

        sprintf( szExec, "INSERT INTO records( call_ssid ) VALUES( '%s' )", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( PQresultStatus( pResult ) != PGRES_COMMAND_OK )
        {
            PQclear( pResult );
            PQfinish( pConn );
            return;
        }
        PQclear( pResult );

        // since record did not exist, write all data as record values

        // temp, wind
        sprintf( szExec, "UPDATE records SET htemp=%i, htemp_date='%s', ltemp=%i, ltemp_date='%s', "
            "wgh_spd=%i, wgh_dir=%i, wgh_date='%s' WHERE call_ssid='%s'",
            g_Local.Data.thr.val, szDate, g_Local.Data.tlr.val, szDate,
            g_Local.Data.wgh.spd, g_Local.Data.wgh.dir, szDate, g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );

        // precip
        sprintf( szExec, "UPDATE records SET precip24=%i, precip24_date='%s', precipmh=%i, precipmh_date='%s', "
            "precipyh=%i, precipyh_date='%s' WHERE call_ssid='%s'",
            g_Local.Data.pca, szDate, g_Local.Data.ptm, szDate, g_Local.Data.pta, szDate, g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        PQclear( pResult );

        // barometer
        if( g_Sensors.bBarom || g_Sensors.bOwBarom )
        {
            sprintf( szExec, "UPDATE records SET bhr=%i, bhr_date='%s', blr=%i, blr_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.bhr.val, szDate, g_Local.Data.blr.val, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
            PQclear( pResult );
        }

        // emp
        if( g_Sensors.bOwEmp )
        {
            sprintf( szExec, "UPDATE records SET hemp=%i, hemp_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.edc, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
            PQclear( pResult );
        }

        // radiation
        if( g_Sensors.bOwRad )
        {
            sprintf( szExec, "UPDATE records SET hrad=%i, hrad_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.rmax, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
            PQclear( pResult );
        }

        pResult = PQexec( pConn, "COMMIT" );
        PQclear( pResult );

        PQfinish( pConn );

        return;

    }   // end if record does not exist

    PQclear( pResult );

    //////////////////////////////////////////// record exists

    // high temp
    sprintf( szExec, "SELECT htemp FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.thr.val >= nTemp )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET htemp=%i, htemp_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.thr.val, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    // low temp
    sprintf( szExec, "SELECT ltemp FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.tlr.val <= nTemp )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET ltemp=%i, ltemp_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.tlr.val, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    // high gust
    sprintf( szExec, "SELECT wgh_spd FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.wgh.spd >= nTemp )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET wgh_spd=%i, wgh_dir=%i, wgh_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.wgh.spd, g_Local.Data.wgh.dir, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    ////////////// precip

    // 24 hour precip
    sprintf( szExec, "SELECT precip24 FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.pca >= nTemp )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET precip24=%i, precip24_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.pca, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    // high month
    sprintf( szExec, "SELECT precipmh FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.ptm >= nTemp )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET precipmh=%i, precipmh_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.ptm, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    // month low  - must only reset if first day of new month
    if( g_Time.PrevMonth() != g_Time.Month() )
    {
        sprintf( szExec, "SELECT precipml FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.ptm <= nTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET precipml=%i, precipml_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.ptm, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        else // does not exist
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET precipml=%i, precipml_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.ptm, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
        PQclear( pResult );
    }

    // year high
    sprintf( szExec, "SELECT precipyh FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
    pResult = PQexec( pConn, szExec );
    if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
    {
        nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
        if( g_Local.Data.pca && (g_Local.Data.pta >= nTemp) )
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET precipyh=%i, precipyh_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.pta, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
    }
    PQclear( pResult );

    // year low -  must only reset if first day of new year
    if( g_Time.PrevYear() != g_Time.Year() )
    {
        sprintf( szExec, "SELECT precipyl FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.pta <= nTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET precipyl=%i, precipyl_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.pta, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        else    // does not previously exist
        {
            PQclear( pResult );
            sprintf( szExec, "UPDATE records SET precipyl=%i, precipyl_date='%s' WHERE call_ssid='%s'",
                g_Local.Data.pta, szDate, g_Local.Info.szCallSsid );
            pResult = PQexec( pConn, szExec );
        }
        PQclear( pResult );
    }

    ////////////// barometric pressure

    if( g_Sensors.bBarom || g_Sensors.bOwBarom )
    {

        // high barometer
        sprintf( szExec, "SELECT bhr FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.bhr.val >= nTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET bhr=%i, bhr_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.bhr.val, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        PQclear( pResult );

        // low barometer
        sprintf( szExec, "SELECT blr FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.blr.val <= nTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET blr=%i, blr_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.blr.val, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        PQclear( pResult );

    }   // g_Sensors.bBarom || g_Sensors.bOwBarom

    if( g_Sensors.bOwEmp )
    {
        // high emp
        sprintf( szExec, "SELECT hemp FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            uTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.edc >= uTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET hemp=%i, hemp_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.edc, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        PQclear( pResult );

    }   // g_Sensors.bOwEmp

    if( g_Sensors.bOwRad )
    {
        // high radiation count
        sprintf( szExec, "SELECT hrad FROM records WHERE call_ssid='%s'", g_Local.Info.szCallSsid );
        pResult = PQexec( pConn, szExec );
        if( (PQresultStatus( pResult ) == PGRES_TUPLES_OK) && PQntuples( pResult ) )
        {
            nTemp = atoi( PQgetvalue( pResult, 0, 0 ) );
            if( g_Local.Data.rmax >= nTemp )
            {
                PQclear( pResult );
                sprintf( szExec, "UPDATE records SET hrad=%i, hrad_date='%s' WHERE call_ssid='%s'",
                    g_Local.Data.rmax, szDate, g_Local.Info.szCallSsid );
                pResult = PQexec( pConn, szExec );
            }
        }
        PQclear( pResult );

    }   // g_Sensors.bOwRad

    PQfinish( pConn );
}

