/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _ARCHIVE_H_
#define _ARCHIVE_H_

#include <time.h>

// libwxn
#include <datastruct.h>


//////////////////////////////////////////////////////////////////////////////
// HOURLY - structure of readings taken at the top of each hour.

struct hourly
{
    // int year;
    // int month;
    // int day;
    // int hour;

    int tcr;
    struct wind wac;
    unsigned bar;
    int rlh;
    int hcr;
    unsigned emp;
    int uvc;
    int ist;
    int ish;
    int tsd;
};

#define HOURLY_SIZE sizeof(struct hourly)


////////////////////////////////////////////////////////////////////
// DAILY - structure of daily summary recorded at midnight.

struct daily
{
    // int year;
    // int month;
    // int day;

    struct type2 thr;       // high temp
    struct type2 tlr;       // low temp
    int mtemp;              // mean temp
    struct type2 bhr;       // high barometer
    struct type2 blr;       // low barometer
    unsigned mbar;          // mean barometer
    struct wind2 wgh;       // high wind gust
    struct wind2 wgl;       // low wind gust
    struct wind mwind;      // wind mean
    struct type2 hhr;       // high humidity
    struct type2 hlr;       // low humidity
    int mhumid;             // mean humidity
    int precip;             // total precip
    unsigned edc;           // total emp hits
    struct type2 uvh;       // high UV reading
    struct type2 uvl;       // low UV reading
    int nHeatDays;          // heating degree days
    int nCoolDays;          // cooling degree days
};

#define     DAILY_SIZE      sizeof(struct daily)

struct iavg
{
    int val;
    int cnt;
};

struct ulavg
{
    unsigned long val;
    int cnt;
};

struct wavg
{
    int spd;
    int dir;
    int cnt;
};

////////////////////////////////////////////////////////////////////
// MONTHLY - structure of monthly average/summary

struct monthly
{
    struct iavg mtemp;  // mean temperature
    int nHeatDays;      // total heating degree days
    int nCoolDays;      // total cooling degree days
    struct wavg mwind;  // wind average speed/direction
    struct ulavg mbar;  // mean barometric pressure
    struct iavg mhumid; // mean humidity
    int precip;         // total precip
    unsigned long emp;  // total emp hits
    struct iavg muv;    // avg daily UV
    struct iavg mrad;   // avg daily background radiation count
    struct iavg mtsd;   // avg daily soil temp at 6" depth
};

#define     MONTHLY_SIZE    sizeof(struct monthly)

////////////////////////////////////////////////////////////////////
// Record Value Types

struct recdate
{
    int month;
    int day;
    int year;
};

struct rect         // generic values
{
    int val;
    struct recdate date;
};

struct rect2        // wind records require secondary value
{
    int spd;
    int dir;
    struct recdate date;
};


////////////////////////////////////////////////////////////////////
// weather record data

struct record
{
    struct rect  htemp;             // temperature
    struct rect  ltemp;
    struct rect2 hwind;
    struct rect  hbar;              // barometer
    struct rect  lbar;
    struct rect  precip24;          // highest 24 hour precip
    struct rect  precipmh;          // high/low precip in a month
    struct rect  precipml;
    struct rect  precipyh;          // high/low precip in a year
    struct rect  precipyl;
    struct rect  emp;               // max emp hits in 24 hours
    struct rect  rad;               // highest avg radiation count
    struct rect  uv;                // highest UV reading recorded
};


// prototypes

void WriteHourly( void );
void WriteDaily( void );
void WriteMonthly( void );
void SetWxRecords( void );


#endif      // _ARCHIVE_H_

