/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// email.cpp - sends email on local host
//

#define _EMAIL_CPP_

#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <time.h>

// libwxn
#include <logmsg.h>

#include "email.h"
#include "mresolv.h"


#ifndef FOREVER
    #define FOREVER for(;;)
#endif

#ifndef BOOL
    typedef int BOOL;
#endif

#ifndef TRUE
    #define TRUE  1
    #define FALSE 0
#endif


BOOL email( mailform & Mail )
{
    struct sockaddr_in address;
    int hSock;
    char szBuffer[1024];
    int nLen;
    BOOL bStatus;
    struct tm * pTm;
    time_t tTime;

    memset( &address, 0, sizeof address );
    address.sin_family = AF_INET;
    if( IsIP( Mail.sToHost ) )
    {
        if( !inet_aton( Mail.sToHost, &address.sin_addr ) )
            return FALSE;
    }
    else    // is in domain-name form
    {
        if( !ResolveMXDomain( &Mail ) )
            return FALSE;
        memcpy( &address.sin_addr, &Mail.inaddr, 4 );
    }
    address.sin_port = htons( 25 );
    nLen = sizeof( address );

    hSock = socket( AF_INET, SOCK_STREAM, 0 );
    if( connect( hSock, (struct sockaddr *)&address, nLen ) == -1 )
    {
        g_log.PostMsg( HOSTMSG_SOCKET_FAIL, "email" );
        return FALSE;
    }

    FOREVER     // no looping here - just a convenience to exit in case of failure
    {
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "220" ) )
        {
            bStatus = FALSE;
            break;
        }

        // preamble
        sprintf( szBuffer, "HELO %s\r\n", Mail.sFromHost );
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "250" ) )
        {
            bStatus = FALSE;
            break;
        }

        sprintf( szBuffer, "MAIL FROM: <%s@%s>\r\n", Mail.sFromAcct, Mail.sFromHost );
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "250" ) )
        {
            bStatus = FALSE;
            break;
        }

        sprintf( szBuffer, "RCPT TO: <%s@%s>\r\n", Mail.sToAcct, Mail.sToHost );
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "250" ) )
        {
            bStatus = FALSE;
            break;
        }

        sprintf( szBuffer, "DATA\r\n" );
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "354" ) )
        {
            bStatus = FALSE;
            break;
        }

        // headers
        tTime = time(NULL);
        pTm = localtime( &tTime );
        strftime( szBuffer, 1024, "Date: %a, %d %b %Y %H:%M:%S %Z\r\n", pTm );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        sprintf( szBuffer, "From: %s <%s@%s>\r\n", Mail.sFromName, Mail.sFromAcct, Mail.sFromHost );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        sprintf( szBuffer, "To: %s <%s@%s>\r\n", Mail.sToName, Mail.sToAcct, Mail.sToHost );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        sprintf( szBuffer, "Reply-To: %s <%s>\r\n", Mail.sReplyTo, Mail.sReplyTo );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        sprintf( szBuffer, "Subject: %s\r\n", Mail.sSubject );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        sprintf( szBuffer, "\r\n" );
        send( hSock, szBuffer, strlen(szBuffer), 0 );

        // body
        send( hSock, Mail.pBody, Mail.nBodyLen, 0 );

        //clean-up
        sprintf( szBuffer, "\r\n.\r\n" );    // end data
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
        {
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
            bStatus = FALSE;
            break;
        }
        if( !strstr( szBuffer, "250" ) )
        {
            bStatus = FALSE;
            break;
        }

        sprintf( szBuffer, "QUIT\r\n" );    // terminate
        send( hSock, szBuffer, strlen(szBuffer), 0 );
        if( recv( hSock, szBuffer, 1024, 0 ) == -1 )
            g_log.PostMsg( HOSTMSG_SOCKET_RCV_FAIL, "email" );
        if( !strstr( szBuffer, "221" ) )
        {
            bStatus = FALSE;
            break;
        }

        bStatus = TRUE;
        break;
    }

    close( hSock );

    return bStatus;
}

