/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _UMX_H_
#define _UMX_H_

#ifndef BOOL
    typedef int BOOL;
#endif

// timeout in seconds for ultimeter stations

#define UMX_TIMEOUT     15

// length of headers for Ultimeter stations

#define CR_HEADER 4         // complete record
#define PK_HEADER 5         // packet
#define DL_HEADER 2         // data logging
#define U2_HEADER 1         // Ultimeter II

// length of data records for Ultimeter stations

#define RL1 1
#define RL2 2
#define RL4 4

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// data line lengths from Ultimeter stations

// Ultimter II

#define UM2CNT  14         // 13 hex digits plus '*' or '#' header

// Ultimeter 100, 500, 800, 2000

#define U2KLEN1 42         // 40 hex digits plus '!!' header
#define U2KLEN2 46         // 44  "    "     "    "     "
#define U2KLEN3 50         // 48  "    "     "    "     "

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Offsets for record types in Data Logging mode

// Ultimeter II

#define UM2_270     12      // 270 degrees
#define UM2_90      4       // 90 degrees
#define UM2_360     16      // 360 degrees

#define UM2_DIR     1
#define UM2_SPEED   2
#define UM2_TEMP    4
#define UM2_PRECIP  6       // "upper" precip

// Ultimeter 100, 500, 800, 2000

#define U2K_270     192     // 270 degrees
#define U2K_90      64      // 90 degrees
#define U2K_360     256     // 360 degrees

#define U2K_HEADER "!!"

#define WSPD_OFF    DL_HEADER+0
#define WDIR_OFF    DL_HEADER+RL4
#define OTMP_OFF    DL_HEADER+(2*RL4)
#define RAIN_OFF    DL_HEADER+(3*RL4)
#define BARM_OFF    DL_HEADER+(4*RL4)
#define ITMP_OFF    DL_HEADER+(5*RL4)
#define OHUM_OFF    DL_HEADER+(6*RL4)
#define IHUM_OFF    DL_HEADER+(7*RL4)
#define DATE_OFF    DL_HEADER+(8*RL4)
#define TIME_OFF    DL_HEADER+(9*RL4)

// Complete Record Mode

#define BCR_OFF  7*RL4

#define BHRp_OFF 77*RL4
#define BHRt_OFF 78*RL4

#define BLRp_OFF 31*RL4
#define BLRt_OFF 32*RL4

#define TCR_OFF   7*RL4

#define THRp_OFF 68*RL4
#define THRt_OFF 69*RL4

#define TLRp_OFF 24*RL4
#define TLRt_OFF 25*RL4

#define HCR_OFF  12*RL4

#define HHRp_OFF 91*RL4
#define HHRt_OFF 92*RL4

#define HLRp_OFF 47*RL4
#define HLRt_OFF 48*RL4

#define WACp_OFF  0*RL4
#define WACs_OFF  1*RL4

#define WGHp_OFF  2*RL4
#define WGHs_OFF  3*RL4

#define WCA_OFF   4*RL4

#define RCA_OFF   6*RL4
#define TCS_OFF  11*RL4
#define HCS_OFF  13*RL4


// public functions

BOOL UmxProcessData( void );
BOOL UmxInit( BOOL bVerbose );
void UmxStopThread( BOOL bVerbose );
BOOL UmxDataReady( void );

#endif  // _UMX_H_
