/***************************************************************************
 *   Copyright (C) 2005 by John Bennett   *
 *   n4xi@arrl.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//
// qconfig.cpp
//

#define _QCONFIG_CPP_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>

// libwxn
#include <common.h>
#include <datastruct.h>
#include <logmsg.h>

// other
#include "qconfig.h"
#include "qglobal.h"


// init names, types, variables

#define STRING  0
#define CHAR    1
#define BOOLEAN 2
#define INTEGER 3
#define SHORT   4
#define UNS_INT 5
#define TIME    6
#define LONG    7
#define MULTISZ 8
#define FLINT   9
#define FLOAT   10


struct ini QueryIni[] =
{
    { "UseAlias",           BOOLEAN,    &g_QueryConfig.bUseAlias },
    { "Retries",            INTEGER,    &g_QueryConfig.nRetries },
    { "FrAckTimer",         INTEGER,    &g_QueryConfig.nFrAckTimer },
    { "DelayTime",          INTEGER,    &g_QueryConfig.nDelayTime }
};

#define QUERY_INI_ENTRIES (sizeof(QueryIni)/ sizeof(struct ini))


/////////////////////////////////////////////////////////////////////////////
// ReadQueryConfig - read <wxnqueryd.conf> to set required parameters.
// Returns TRUE success, FALSE failure.
//

BOOL ReadQueryConfig( void )
{
    FILE * hFile;
    BOOL bFound;
    BOOL bNoMatch;
    int i;
    char szString[STRING_SIZE];
    char szLine[STRING_SIZE];
    char szMsg[STRING_SIZE];
    char * pS;
    char * pParam;
    char * pVal;
    double dVal;

    hFile = fopen( WXN_SYSCONFDIR "wxnqueryd.conf", "r" );
    if( hFile == NULL )
    {
        g_log.PostMsg( QRYMSG_WXNQUERYD_CONF_FAIL, "cannot open/read file" );
        return FALSE;
    }

    // set defaults
    memset( &g_QueryConfig, 0, sizeof(struct queryconf) );
    g_QueryConfig.nRetries = 5;
    g_QueryConfig.nFrAckTimer = 10;

    FOREVER // for(;;)
    {
        if( fgets( szLine, STRING_SIZE, hFile ) == NULL )
            break;      // EOF

        if( *szLine == '\r' || *szLine == '\n' )
            continue;               // empty line

        pS =strchr( szLine, 0 );
        if( *(pS-1) == '\r' || *(pS-1) == '\n' )
            *(pS-1) = 0;

        strcpy( szString, szLine );

        bFound = FALSE;
        bNoMatch = TRUE;

        pParam = szString;
        while( *pParam )
        {
            if( *pParam != ' ' )
                break;
            pParam++;
        }

        if( *pParam == '#' )      // comment
            continue;

        pVal = pParam;
        while( *pVal )
        {
            if( *pVal == ' ' )
            {
                *pVal++ = 0;        // terminate the parameter portion of string
                continue;
            }
            if( *pVal++ == '=' )
                break;
        }
        while( *pVal )
        {
            if( *pVal != ' ' )
                break;
            pVal++;
        }

        if( *pVal == '"' )          // check for enclosing quotes
            pVal++;
        if( strchr( pVal, '"' ) )
            *strchr( pVal, '"' ) = 0;

        if( *pVal == 0 )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( QRYMSG_WXNQUERYD_CONF_FAIL, szMsg );
            continue;
        }

        for( i=0; i<(int)QUERY_INI_ENTRIES; i++ )
        {
            if( strcasecmp( pParam, QueryIni[i].szParam ) == 0 )
            {
                bFound = TRUE;
                bNoMatch = FALSE;
                if( pVal == NULL )
                    break;
                switch( QueryIni[i].nType )
                {
                    case STRING:
                        strcpy( (char *) QueryIni[i].pType, pVal );
                        break;

                    case CHAR:
                        *(char *) QueryIni[i].pType = *pVal;
                        break;

                    case BOOLEAN:
                        if( ((*pVal & 0x005f) == 'T') || ((*pVal & 0x005f) == 'Y') )
                            *((BOOL *) QueryIni[i].pType) = TRUE;
                        else
                            *((BOOL *) QueryIni[i].pType) = FALSE;
                        break;

                    case INTEGER:
                        *((int *) QueryIni[i].pType) = atoi( pVal );
                        break;

                    case SHORT:
                        *((short *) QueryIni[i].pType) = (short) atoi( pVal );
                        break;

                    case UNS_INT:
                        *((unsigned *) QueryIni[i].pType) = (unsigned) atol( pVal );
                        break;

                    case TIME:
                        *((time_t *) QueryIni[i].pType) = (time_t) atol( pVal );
                        break;

                    case LONG:
                        *((long *) QueryIni[i].pType) = atol( pVal );
                        break;

                    case FLINT:
                        dVal = atof( pVal ) * 100;
                        *((int *) QueryIni[i].pType) = (int) dVal;   // one-liner introduces error
                        break;

                    case FLOAT:
                        dVal = atof( pVal );
                        *((double *) QueryIni[i].pType) = dVal;
                        break;

                    default:
                        bNoMatch = TRUE;
                        break;
                }
            }

            if( bFound )
                break;
        }

        if( bNoMatch )
        {
            sprintf( szMsg, "don't understand <%s>", szLine );
            g_log.PostMsg( QRYMSG_WXNQUERYD_CONF_FAIL, szMsg );
        }
    }

    if( g_QueryConfig.nDelayTime == 0 || g_QueryConfig.nDelayTime>60 || g_QueryConfig.nDelayTime < 5 )
        g_QueryConfig.nDelayTime = 30;

    fclose( hFile );

    return TRUE;
}

